<?php

namespace Filoblu\Refilo\Model\Adapter;

use Filoblu\Refilo\Helper\Data as HelperData;
use MongoDB\Driver\BulkWrite;
use MongoDB\Driver\Command;
use MongoDB\Driver\Manager;

class Mongo extends AbstractAdapter
{
    /**
     * @var HelperData
     */
    private $helperData;

    /**
     * Mongo constructor.
     * @param HelperData $helperData
     */
    public function __construct(
        HelperData $helperData
    ) {
        $this->helperData = $helperData;
        $this->connect();
    }

    public function connect()
    {
       // $this->_connection = new Manager($this->helperData->getConfigValue("mongo/general/mongo_host"));
        $this->_connection = new Manager("mongodb://35.205.137.140:27018");
    }

    /**
     * @param $key
     * @param $objects
     * @param $store
     */
    public function push($key, $objects, $collection)
    {
        $bulk = new BulkWrite();

        foreach ($objects as $object) {
            try {
                $bulk->update(["_id"=>$object->{$key}], $object, ['multi' => false, 'upsert' => true]);
            } catch (MongoDB\Driver\Exception\Exception $e) {
                echo $e->getMessage();
            }
        }

        try {
            $database = $this->helperData->getConfigValue("mongo/general/mongo_database");
            $result = $this->_connection->executeBulkWrite($collection, $bulk);
            printf("Inserted %d document(s)\n", $result->getInsertedCount());
            printf("Matched  %d document(s)\n", $result->getMatchedCount());
            printf("Updated  %d document(s)\n", $result->getModifiedCount());
            printf("Upserted %d document(s)\n", $result->getUpsertedCount());
            printf("Deleted  %d document(s)\n", $result->getDeletedCount());
        } catch (MongoDB\Driver\Exception\Exception $e) {
            echo $e->getMessage();
        }
    }

    /**
     * @param $key
     * @param $objects
     * @param $collection
     * @return mixed|void
     */
    public function update($key, $objects, $collection)
    {
        $bulk = new BulkWrite();

        foreach ($objects as $object) {
            try {
                $bulk->update(["_id"=>$object->{$key}], ['$set' => $object], ['multi' => false, 'upsert' => false]);
            } catch (MongoDB\Driver\Exception\Exception $e) {
                echo $e->getMessage();
            }
        }

        try {
            $database = $this->helperData->getConfigValue("mongo/general/mongo_database");
            $result = $this->_connection->executeBulkWrite($collection, $bulk);
            printf("Inserted %d document(s)\n", $result->getInsertedCount());
            printf("Matched  %d document(s)\n", $result->getMatchedCount());
            printf("Updated  %d document(s)\n", $result->getModifiedCount());
            printf("Upserted %d document(s)\n", $result->getUpsertedCount());
            printf("Deleted  %d document(s)\n", $result->getDeletedCount());
        } catch (MongoDB\Driver\Exception\Exception $e) {
            echo $e->getMessage();
        }
    }

    /**
     * @return mixed
     */
    public function ensureIndex($store)
    {
        try {
            $database = $this->helperData->getConfigValue("mongo/general/mongo_database");
            $command = new Command([
                "createIndexes" => "products_" . $store,
                "indexes"       => [
                    [
                        "name" => "categories",
                        "key"  => [ "categories" => 1],
                        "ns"   => "$database.products_" . $store,
                    ],
                    [
                        "name" => "filters",
                        "key"  => [ "filters" => 1],
                        "ns"   => "$database.products_" . $store,
                    ],
                    [
                        "name" => "pricing",
                        "key"  => [ "pricing" => 1],
                        "ns"   => "$database.products_" . $store,
                    ],
                    [
                        "name" => "childs",
                        "key"  => [ "childs" => 1],
                        "ns"   => "$database.products_" . $store,
                    ]
                ],
            ]);
            $result = $this->_connection->executeCommand($database, $command);
            printf("added Indexes for %s \n", "$database.products_" . $store);
        } catch (MongoDB\Driver\Exception\Exception $e) {
            echo $e->getMessage();
        }
    }
}
