<?php

namespace Filoblu\Refilo\Model\Indexer\Core;

use Filoblu\Refilo\Model\Indexer\Core\Provider\QueryProviderInterface;
use Magento\Framework\App\ResourceConnection;
use Zend_Db;

/**
 * Class AbstractSqlToMongoAdapter
 * @package Filoblu\Refilo\Model\Indexer\Core
 */
abstract class AbstractSqlToMongoAdapter implements SqlToMongoAdapterInterface
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;
    /**
     * @var ClassNameToSqlInterface
     */
    private $classNameToSql;

    /**
     * AbstractSqlToMongoAdapter constructor.
     * @param ResourceConnection $resourceConnection
     * @param ClassNameToSqlInterface $classNameToSql
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        ClassNameToSqlInterface $classNameToSql
    ) {
        $this->resourceConnection = $resourceConnection;
        $this->classNameToSql = $classNameToSql;
    }

    /**
     * @param QueryProviderInterface $provider
     * @return array
     */
    public function adaptFromSql(QueryProviderInterface $provider) : array
    {
        $query = sprintf(
            'SELECT \'%s\' AS filoblu_adapt_class,filoblu_adapt_query.* FROM (%s) AS filoblu_adapt_query',
            $this->classNameToSql->get($this->getMongoModel()),
            $provider->getQuery()
        );
        return $this->resourceConnection->getConnection()
            ->fetchAll($query, $provider->getBindings(), Zend_Db::FETCH_CLASSTYPE);
    }

    /**
     * @return string
     */
    abstract public function getMongoModel(): string;

}
