<?php

namespace Filoblu\Refilo\Model\Indexer\Core;

use Filoblu\Refilo\Model\Adapter\Mongo;
use Filoblu\Refilo\Model\Indexer\Core\Provider\QueryProviderInterface;
use Filoblu\Refilo\Model\Indexer\Core\Provider\QueryProviderInterfaceFactory;
use Magento\Store\Model\StoreManagerInterface;

class BaseWebsiteProcessor extends BaseMongoIndexer
{
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * BaseWebsiteProcessor constructor.
     * @param SqlToMongoAdapterInterface $sqlToMongoAdapter
     * @param QueryProviderInterfaceFactory $queryProviderFactory
     * @param StoreManagerInterface $storeManager
     * @param Mongo $mongo
     * @param string $mongoKey
     * @param string $mongoCollection
     */
    public function __construct(
        SqlToMongoAdapterInterface $sqlToMongoAdapter,
        QueryProviderInterfaceFactory $queryProviderFactory,
        StoreManagerInterface $storeManager,
        Mongo $mongo,
        string $mongoKey,
        string $mongoCollection
    ) {
        parent::__construct(
            $sqlToMongoAdapter,
            $queryProviderFactory,
            $mongo,
            $mongoKey,
            $mongoCollection
        );
        $this->storeManager = $storeManager;
    }

    protected function performAction($ids = null)
    {
        foreach ($this->storeManager->getStores() as $store) {
            /** @var QueryProviderInterface $provider */
            $provider = $this->queryProviderFactory->create();
            if($ids){
                $provider->filter($ids);
            }
            $provider->website($store->getWebsiteId());
            $result = $this->sqlToMongoAdapter->adaptFromSql($provider);
            $this->mongo->update(
                $this->getMongoKey(),
                $result,
                sprintf($this->getTargetMongoCollection(), $store->getId())
            );
        }
    }
}
