<?php


namespace Filoblu\Refilo\Model\Indexer\Processor;
use Filoblu\Refilo\Model\Indexer\Processor\Type\Configurable;
use Filoblu\Refilo\Model\Indexer\Processor\Type\Grouped;
use Filoblu\Refilo\Model\Indexer\Processor\Type\Bundle;
use Magento\Catalog\Api\Data\ProductSearchResultsInterface;
use Magento\Catalog\Model\Product\Attribute\Source\Status;
use Magento\Catalog\Model\Product\Visibility;
use Magento\Catalog\Model\ProductRepository;
use Magento\Framework\Api\FilterBuilder;
use Magento\Framework\Api\Search\FilterGroup;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Api\SearchCriteriaInterface;
use \Magento\Catalog\Model\ResourceModel\Product\Attribute\CollectionFactory as AttributeCollection;
use \Filoblu\Refilo\Model\Indexer\Processor\Type\Simple;
use \Magento\Framework\App\ResourceConnection;

/**
 * @property  searchCriteria
 */
class Catalog
{

    /**
     * @var int
     */
    private $totalPages = 0;

    /**
     * @var Attribute
     */
    private $_product_attributes;

    /**
     * @var ProductRepository
     */
    private $productRepository;
    /**
     * @var SearchCriteriaInterface
     */
    private $criteria;
    /**
     * @var FilterGroup
     */
    private $filterGroup;
    /**
     * @var FilterBuilder
     */
    private $filterBuilder;
    /**
     * @var Status
     */
    private $productStatus;
    /**
     * @var Visibility
     */
    private $productVisibility;
    /**
     * @var Filoblu\Refilo\Model\Indexer\Processor\Type\Simple|Simple
     */
    private $_simple_product;
    /**
     * @var Filoblu\Refilo\Model\Indexer\Processor\Type\Configurable|Configurable
     */
    private $_configurable_product;
    /**
     * @var Filoblu\Refilo\Model\Indexer\Processor\Type\Grouped|Grouped
     */
    private $_grouped_product;
    /**
     * @var Filoblu\Refilo\Model\Indexer\Processor\Type\Bundle|Bundle
     */
    private $_bundle_product;
    /**
     * @var ResourceConnection
     */
    private $connection;
    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;

    /**
     * @var array
     */
    protected $_options;


    /**
     * Catalog constructor.
     * @param AttributeCollection $_product_attributes
     * @param ProductRepository $productRepository
     * @param SearchCriteriaInterface $criteria
     * @param FilterGroup $filterGroup
     * @param FilterBuilder $filterBuilder
     * @param Status $productStatus
     * @param Visibility $productVisibility
     * @param Simple $simple_product
     * @param Configurable $configurable_product
     * @param Bundle $bundle_product
     * @param Grouped $grouped_product
     * @param ResourceConnection $connection
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     */
    public function __construct(

        AttributeCollection $_product_attributes,
        ProductRepository $productRepository,
        SearchCriteriaInterface $criteria,
        FilterGroup $filterGroup,
        FilterBuilder $filterBuilder,
        Status $productStatus,
        Visibility $productVisibility,
        Simple $simple_product,
        Configurable $configurable_product,
        Bundle $bundle_product,
        Grouped $grouped_product,
        ResourceConnection $connection,
        SearchCriteriaBuilder $searchCriteriaBuilder
    )
    {

        $this->connection = $connection->getConnection();
        $attribute_collection = $_product_attributes->create();
        $this->_product_attributes = $this->decodeAttibutes($attribute_collection);

        $this->productRepository = $productRepository;
        $this->criteria = $criteria;
        $this->filterGroup = $filterGroup;
        $this->filterBuilder = $filterBuilder;
        $this->productStatus = $productStatus;
        $this->productVisibility = $productVisibility;
        $swatch_options = $this->processSwatchOptions();

        $this->_simple_product = $simple_product;
        $this->_simple_product->setProductAttributes($this->_product_attributes);
        $this->_simple_product->setSwatchOptions($swatch_options);

        $this->_configurable_product = $configurable_product;
        $this->_configurable_product->setProductAttributes($this->_product_attributes);
        $this->_configurable_product->setSwatchOptions($swatch_options);

        $this->_bundle_product = $bundle_product;
        $this->_bundle_product->setProductAttributes($this->_product_attributes);
        $this->_bundle_product->setSwatchOptions($swatch_options);

        $this->_grouped_product = $grouped_product;
        $this->_grouped_product->setProductAttributes($this->_product_attributes);
        $this->_grouped_product->setSwatchOptions($swatch_options);

        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
    }

//TODO parent product with partial reindex
    /**
     * @param $store
     * @return array
     */
    public function processOptions($store) {

        $query = "SELECT eav_attribute_option_value.* , eav_attribute_option.attribute_id FROM refilo_mage.eav_attribute_option_value
INNER JOIN eav_attribute_option on eav_attribute_option.option_id = eav_attribute_option_value.option_id where store_id = 0 or store_id = $store ORDER BY store_id ASC;";

        $_options = [];
        $_result = $this->connection->fetchAssoc($query);

        foreach ($_result as $option) {

            $_options[$option["option_id"]] = $option;

        }

        return $_options;

    }

    /**
     * @return array
     */
    public function processSwatchOptions() {

        $query = "SELECT swatch_id, option_id, store_id, `type`, value FROM refilo_mage.eav_attribute_option_swatch";

        $_swatch_options = [];

        $result = $this->connection->fetchAssoc($query);

        foreach ($result as $swatch) {

            $_swatch_options[$swatch['option_id']] = $swatch;

        }

        return $_swatch_options;

    }

    public function decodeAttibutes($attribute_collection) {

        $attribute_array = [];

        foreach ($attribute_collection as $attribute) {

                $attribute_array[$attribute->getData('attribute_code')] = $attribute->getData();
        }

        return $attribute_array;

    }

    /**
     * @param $store
     * @param $batch_size
     * @param $batch_sequence
     * @param bool $ids
     * @return mixed
     */
    public function getProducts($store,$batch_size,$batch_sequence, $ids = false)
    {

        if ($this->_options === null) $this->_options = $this->processOptions($store);

        $productCollection = $this->prepareProductCollection($store,$batch_size,$batch_sequence,$ids);
        return ($productCollection) ? $this->processProductCollection($productCollection) : false;
    }

    /**
     * @param $store
     * @param $batch_size
     * @param $batch_sequence
     * @param bool $ids
     * @return ProductSearchResultsInterface
     */
    protected function prepareProductCollection($store,$batch_size,$batch_sequence, $ids = false)
    {

        if (($this->totalPages < $batch_sequence) && $this->totalPages != 0) {
            return false;
        }

        $this->searchCriteriaBuilder
            ->addFilter('status',$this->productStatus->getVisibleStatusIds(),'in')
//            ->addFilter('visibility',$this->productVisibility->getVisibleInSiteIds(),'in')
            ->addFilter('store_id',$store,'eq');

          if (is_array($ids)) {
              $this->searchCriteriaBuilder->addFilter('entity_id',$ids,'in');
          }

        $searchCriteriaBuilder =  $this->searchCriteriaBuilder->create();

        $searchCriteriaBuilder->setPageSize($batch_size);
        $searchCriteriaBuilder->setCurrentPage($batch_sequence);
        printf("Prepare Collection\n\r");
        $collection = $this->productRepository->getList($searchCriteriaBuilder);

        if (!$this->totalPages) {
            $count = $collection->getTotalCount();
            $this->totalPages = ceil($count / $batch_size);

        }
        printf("Page %d of %d \n\r",$batch_sequence,$this->totalPages);
        return $collection;
    }

    /**
     * @param $_product_collection
     * @return array
     */
    protected function processProductCollection($_product_collection) {

        $items = [];
        foreach ($_product_collection->getItems() as $product) {
            switch ($product->getData('type_id')) {
                case "simple":
                    $this->_simple_product->setProduct($product);
                    $this->_simple_product->setOptions($this->_options);
                    $item = $this->_simple_product->process();
                    $items[] = $item;
                    break;
                case "configurable":
                    $this->_configurable_product->setProduct($product);
                    $this->_configurable_product->setOptions($this->_options);
                    $this->_configurable_product->process();
                    //$child_items = $this->_configurable_product->getChildProducts();
                    $items[] = $this->_configurable_product->getItem();
                    //$items = array_merge($items,$child_items);
                    unset($child_items);
                    break;
                case "bundle":
                    $this->_bundle_product->setProduct($product);
                    $this->_bundle_product->setOptions($this->_options);
                    $this->_bundle_product->process();
                    $items[] = $this->_bundle_product->getItem();
                    unset($child_items);
                    break;
                case "grouped":
                    $this->_grouped_product->setProduct($product);
                    $this->_grouped_product->setOptions($this->_options);
                    $this->_grouped_product->process();
                    $items[] = $this->_grouped_product->getItem();
                    unset($child_items);
                    break;
            }
            unset($product);
        }
        $this->productRepository->cleanCache();
        return $items;


    }

    /**
     * @return int
     */
    public function getTotalPages()
    {
        return $this->totalPages;
    }

    /**
     * @param int $total_pages
     */
    public function setTotalPages($totalPages)
    {
        $this->totalPages = $totalPages;
    }

}
