<?php

namespace Filoblu\Refilo\Model\Indexer\Provider;

use Filoblu\Refilo\Model\Indexer\Core\Provider\QueryProviderInterface;
use Magento\Framework\App\ResourceConnection;
use Zend_Db;

class StockProvider implements QueryProviderInterface
{
    /** @var null  */
    protected $ids = null;
    /** @var int */
    protected $website;
    /** @var ResourceConnection  */
    private $resourceConnection;
    /** @var array  */
    private $bindings = [];

    /**
     * StockProvider constructor.
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        ResourceConnection $resourceConnection
    ) {
        $this->resourceConnection = $resourceConnection;
    }

    /**
     * @return string
     */
    public function getQuery(): string
    {
        $connection = $this->resourceConnection->getConnection();
        $inventoryStock = $connection->getTableName('inventory_stock');

        $select = $connection->select();

        foreach ($connection->fetchAll("SELECT * FROM {$inventoryStock}", [], Zend_Db::FETCH_ASSOC) as $stock) {
            $stockId = $stock['stock_id'];
            $inventoryPerStock = $connection->getTableName("inventory_stock_{$stockId}");

            $select->from(
                $inventoryPerStock,
                [
                    '_id' => 'product_id',
                    'website_id' => 'website_id',
                    'quantity' => 'quantity',
                    'is_salable' => 'is_salable'
                ]
            );
            if ($this->ids !== null) {
                $select->where('product_id IN (' . implode(',', $this->ids) . ')');
            }

            if ($this->website !== null) {
                $select->where("website_id = {$this->website}");
            }
            // $this->bindings[':ids'] = ;
        }
        return $select->__toString();
    }

    /**
     * @return array
     */
    public function getBindings(): array
    {
        return $this->bindings;
    }

    /**
     * @param array $ids
     * @return QueryProviderInterface
     */
    public function filter(array $ids): QueryProviderInterface
    {
        $this->ids = $ids;
        return $this;
    }

    public function website(int $website_id): QueryProviderInterface
    {
        $this->website = $website_id;
        return $this;
    }
}
